# A Google Cloud Storage Emulator

This is an emulator for the Google Cloud Storage (GCS) JSON API (both over
REST and gRPC), and a limited support for the XML API. While it emulates most of
the behaviors of GCS, it was developed mostly as an emulator for the client
library. The emulator, for example, performs far fewer error checks and no permission
checks (ACL/IAM). Generally, the error codes are similar to the ones generated by
GCS, but the error messages are not. In short, this is intended for testing, and
not as a general purpose emulator.

## Install Dependencies

```bash
pip install -r requirements.txt
```

## Run Test Bench

### Emulator

```bash
cd google-cloud-cpp
# To make sure `Keep-Alive` header works, `--threads` should not be smaller than 2.
gunicorn --bind "localhost:9000" --worker-class sync --threads 10 --access-logfile - --chdir ./google/cloud/storage/emulator "emulator:run()"
# if you want to use gRPC API, run the following command.
curl "http://localhost:9000/start_grpc?port=8000"
# gRPC server will start listening at port 8000.
```

### Client

For `google-cloud-cpp`, please set the following enviroment variable

```bash
CLOUD_STORAGE_EMULATOR_ENDPOINT=http://localhost:9000 # For JSON and XML API
CLOUD_STORAGE_GRPC_ENDPOINT=localhost:8000 # For gRPC API
```

## Force Failures

You can force the following failures by using the `x-goog-emulator-instructions` header.
The `x-goog-testbench-instructions` header is deprecated, but supported for
backwards compatibility and provides the same functionality as
`x-goog-emulator-instructions`, please change your code to use `x-goog-emulator-instructions` instead.

### return-broken-stream

Set request headers with `x-goog-emulator-instructions: return-broken-stream`.
Emulator will fail after sending 1024*1024 bytes.

### return-corrupted-data

Set request headers with `x-goog-emulator-instructions: return-corrupted-data`.
Emulator will return corrupted data.

### stall-always

Set request headers with `x-goog-emulator-instructions: stall-always`.
Emulator will stall at the beginning.

### stall-at-256KiB

Set request headers with `x-goog-emulator-instructions: stall-at-256KiB`.
Emulator will stall at 256KiB bytes.

### return-503-after-256K

Set request headers with `x-goog-emulator-instructions: return-503-after-256K`.
Emulator will return a `HTTP 503` after sending 256KiB bytes.

### return-503-after-256K/retry-N

Set request headers with `x-goog-emulator-instructions: return-503-after-256K/retry-1` up to `x-goog-emulator-instructions: return-503-after-256K/retry-N`.

For N==1 and N==2 behave like `return-305-after-256K`, for `N>=3` ignore the
failure instruction and return successfully. This is used to test failures during
retry, the client cooperates by sending the retry counter in the failure
instructions.
